#pragma once
#include <cmath>

namespace KapitanGame {
	struct KVector2D {
		float X;
		float Y;

		KVector2D(const float x, const float y) : X(x), Y(y) {}

		float Length() const { return sqrt(X * X + Y * Y); }

		KVector2D operator+(const KVector2D& v2) const {
			return { X + v2.X, Y + v2.Y };
		}

		friend KVector2D& operator+=(KVector2D& v1, const KVector2D& v2) {
			v1.X += v2.X;
			v1.Y += v2.Y;
			return v1;
		}

		KVector2D operator*(const float scalar) const
		{
			return { X * scalar, Y * scalar };
		}

		KVector2D& operator*=(const float scalar) {
			X *= scalar;
			Y *= scalar;
			return *this;
		}

		KVector2D operator-(const KVector2D& v2) const {
			return { X - v2.X, Y - v2.Y };
		}
		friend KVector2D& operator-=(KVector2D& v1, const KVector2D& v2) {
			v1.X -= v2.X;
			v1.Y -= v2.Y;
			return v1;
		}

		KVector2D operator/(const float scalar) const
		{
			return { X / scalar, Y / scalar };
		}
		KVector2D& operator/=(const float scalar) {
			X /= scalar;
			Y /= scalar;
			return *this;
		}

		void Normalize() {
			const float l = Length();
			if (l > 0) {
				(*this) *= 1 / l;
			}
		}
	};
}

