#pragma once
#include <atomic>
#include <SDL_render.h>
#include <vector>

#include "KTexture.h"
#include "KVector2d.h"

namespace KapitanGame {
	class KCircle
	{
	public:
		KCircle(const KVector2D& position, const KVector2D& velocity, const float& radius, const KTexture& texture);

		void CollisionDetectionStep(const std::vector<KCircle>& circles, const bool& shouldSeparate, const bool& shouldReflect);
		void MovementStep(const float& timeStep);
		bool IsCollision(const KCircle& other) const;
		KVector2D GetSeparationVector(const KCircle& other) const;
		void Render(SDL_Renderer* renderer) const;
		void CollisionDetectionWithMapStep(const SDL_Rect& map);
		const int Id;
		static std::atomic<int> IdCounter;
	private:
		KVector2D Position{ 0.f, 0.f };
		KVector2D Velocity{ 0.f, 0.f };
		float Radius{ 1.f };
		const KTexture& Texture;
	};
}


