#include "KCamera.h"

#include "KPawn.h"
#include "Utils.h"

namespace KapitanGame
{
	const SDL_FRect& KCamera::GetViewport() const
	{
		return Viewport;
	}

	float KCamera::GetScale() const
	{
		return Scale;
	}

	const KVector2D& KCamera::GetFocusPoint() const
	{
		return FocusPoint;
	}

	void KCamera::Update(const std::shared_ptr<KPawn>& playerOne, const std::shared_ptr<KPawn>& playerTwo, const SDL_FRect& map)
	{
		FocusPoint = (playerOne->GetPosition() + playerTwo->GetPosition()) * 0.5f;
		Viewport.x = Utils::Clamp(FocusPoint.X - Viewport.w / 2.f, 0.f, map.w - Viewport.w);
		Viewport.y = Utils::Clamp(FocusPoint.Y - Viewport.h / 2.f, 0.f, map.h - Viewport.h);
		PlayArea.x = Utils::Clamp(FocusPoint.X - PlayArea.w / 2.f, static_cast<float>(-Constants::WINDOW_DEAD_ZONE), map.w + static_cast<float>(Constants::WINDOW_DEAD_ZONE));
		PlayArea.y = Utils::Clamp(FocusPoint.Y - PlayArea.h / 2.f, static_cast<float>(-Constants::WINDOW_DEAD_ZONE), map.h + static_cast<float>(Constants::WINDOW_DEAD_ZONE));
	}

	void KCamera::SetDebug(const SDL_FRect& map)
	{
		Viewport = { 0,0,Constants::SCREEN_WIDTH, Constants::SCREEN_HEIGHT };
		Scale = Constants::SCREEN_RATIO > 1.f ? Constants::SCREEN_HEIGHT * 1.f / map.h : Constants::SCREEN_WIDTH * 1.f / map.w;
	}

	const SDL_FRect& KCamera::GetPlayArea() const
	{
		return PlayArea;
	}
}
